const util   = require('util');
const Stream = require('stream');
const types  = require('./types');

/**
 * [MIME description]
 */
function MIME(message){
  Stream.call(this);
  this.buffer  = '';
  this.headers = {};
  this.body    = {};
  for(var k in message){
    this[k] = message[k];
  }
  return this;
};

MIME.CRLF = '\r\n';
MIME.TYPES = types;

/**
 * [PARSE_STATUS description]
 * @type {Object}
 */
MIME.PARSE_STATUS = {
  BODY        : 0x00,
  PART_HEADER : 0x01,
  PART_BODY   : 0x02,
  END         : -1
};

util.inherits(MIME, Stream);

MIME.q = function(address){
  return '<' + address + '>';
};

MIME.kv = function(key, value){
  return [ key, value ].join(': ');
};

MIME.trim = function(s){
  return s.replace(/^"|"$/, '');
}

MIME.filter = function(str){
  return !!str.trim();
};

/**
 * [extension description]
 * @param  {[type]} type [description]
 * @return {[type]}      [description]
 */
MIME.extension = function(type){
  return MIME.TYPES[ type ].extensions;
};

/**
 * [lookup description]
 * @param  {[type]} filename [description]
 * @return {[type]}          [description]
 */
MIME.lookup = function(filename){
  var ext = filename.replace(/.*[\.\/\\]/, '').toLowerCase();
  return Object.keys(MIME.TYPES).filter(function(type){
    var def = MIME.TYPES[ type ];
    return ~(def.extensions||[]).indexOf(ext);
  })[0];
};


var aliaes = {
  from: 'From',
  to  : 'To',
  cc  : 'Cc',
  subject : 'Subject',
};

Object.keys(aliaes).forEach(function(alias){
  MIME.prototype.__defineGetter__(alias, function(){
    return this.headers[ aliaes[alias] ];
  });

  MIME.prototype.__defineSetter__(alias, function(value){
    this.headers[ aliaes[alias] ] = value.toString();
  });
});



/**
 * [function description]
 * @param  {[type]} name  [description]
 * @param  {[type]} value [description]
 * @return {[type]}       [description]
 */
MIME.prototype.header = function(name, value){
  this.headers[ name ] = value;
  return this;
};

/**
 * [write description]
 * @param  {[type]} buf [description]
 * @return {[type]}     [description]
 */
MIME.prototype.write = function(buf){
  this.buffer += buf;
  this.buffer = this.buffer
    .replace(/\r\n/g, '\n')
    .replace(/\n/g, '\r\n');
  var LINE = MIME.CRLF + MIME.CRLF;
  var sp = this.buffer.indexOf(LINE);
  if(sp > -1){
    var header = this.buffer.substr(0, sp);
    this.headers = MIME.parseHeaders(header);
    this.emit('headers', this.headers);
    this.buffer = this.buffer.substr(sp);
  }
  return this;
};

/**
 * [end description]
 * @param  {[type]} buf [description]
 * @return {[type]}     [description]
 */
MIME.prototype.end = function(buf){
  if(buf) this.write(buf);
  var contentType = this.headers[ 'Content-Type' ];
  this.body = MIME.parseBody(this.buffer, contentType);
  this.emit('body', this.body);
  this.emit('end', this);
  return this;
};

/**
 * toString
*/
MIME.prototype.toString = function(){
  var message = [], self = this;
  Object.keys(this.headers).forEach(function(header){
    message.push(MIME.kv(header, self.headers[ header ]));
  });
  message.push(null);
  message.push(this.body._);
  return message.join(MIME.CRLF);
};

/**
 * [parse description]
 * @param  {[type]} content     [description]
 * @param  {[type]} contentType [description]
 * @return {[type]}             [description]
 */
MIME.parse = function(content, contentType){
  var mime = new MIME();
  if(typeof contentType === 'undefined'){
    return mime.end(content);
  }else{
    return MIME.parseBody(content, contentType);
  }
};

/**
 * [parseAddress description]
 * @docs https://tools.ietf.org/html/rfc2822#section-3.4
 * @param  {[type]} address [description]
 * @return {[type]}         [description]
 */
MIME.parseAddress = function(address){
  const r1 = /(.+)@(.+)/;
  const r2 = /^([^<]+)?<(.+)@(.+)>$/;
  if(typeof address !== 'string') 
    throw new TypeError(`[MIME] address must be a string, but got ${address}`);
  if(!r1.test(address))
    throw new SyntaxError(`[MIME] address syntax error: ${address}`);
  var _, name, user, host;
  if(r2.test(address)){
    // Liu song <hi@lsong.org>
    [ _, name, user, host ] = address.match(r2);
  }else{
    // hi@lsong.org
    [ _, user, host ] = address.match(r1);
  }
  host = host.trim();
  user = user.trim();
  name = (name || '').trim();
  return {
    host,
    user,
    name,
    address: `${user}@${host}`,
    toString(){
      name = name ? `"${name}"` : name;
      return `${name}<${this.address}>`;
    }
  };
};
/**
 * [parseHeaders description]
 * @param  {[type]} header [description]
 * @return {[type]}        [description]
 */
MIME.parseHeaders = function(header){
  return header
  .replace(/\n\s+/g, '')
  .split(MIME.CRLF)
  .filter(MIME.filter)
  .map(MIME.parseHeader)
  .reduce(function(item, cur){
    for(var k in cur) item[k] = cur[k];
    return item;
  }, {});
};

/**
 * [parseHeader description]
 * @param  {[type]} str [description]
 * @return {[type]}     [description]
 */
MIME.parseHeader = function(str){
  var h = {};
  var p = str.indexOf(':');
  if(p > -1){
    var k  = str.substr(0, p).trim();
    var v  = str.substr(++p).trim();
    h[ k ] = MIME.parseHeaderValue(v);
  }
  return h;
};

/**
 * [parseHeaderValue description]
 * @param  {[type]} value [description]
 * @return {[type]}       [description]
 */
MIME.parseHeaderValue = function(str){
  var h = {};
  str.split(/;\s?/).map(function(option){
    option = MIME.trim(option);
    var kv = option.match(/^(.+?)=(.*)$/);
    if(kv){
      h[ MIME.trim(kv[1]) ] = MIME.trim(kv[2]);
    }else{
      h._ = option;
    }
  });
  return h;
};

/**
 * [end description]
 * @param  {[type]} buf [description]
 * @return {[type]}     [description]
 */
MIME.parseBody = function(content, contentType){
  if(typeof content !== 'string')
    content = content.toString();
  if(!contentType) return { _: content.trim() };
  if(typeof contentType === 'string'){
    contentType = MIME.parseHeaderValue(contentType);
  }
  var j=-1, h='', body = { _: '' };
  var status = MIME.PARSE_STATUS.BODY;
  var lines = content.split(MIME.CRLF);
  while(lines.length){
    line = lines.shift();
    if(line == '--$--'.replace('$', contentType.boundary)){
      status = MIME.PARSE_STATUS.END;
      break;
    }
    if(line == '--$'.replace('$', contentType.boundary)){
      status = MIME.PARSE_STATUS.PART_HEADER;
      continue;
    }
    switch(status){
      case MIME.PARSE_STATUS.BODY:
        body._ += line;
        break;
      case MIME.PARSE_STATUS.PART_HEADER:
        if(line.trim() == ''){
          status = MIME.PARSE_STATUS.PART_BODY;
          body[ ++j ] = { body: '', headers: MIME.parseHeaders(h) };
          h = '';
          continue;
        }
        h += line + MIME.CRLF;
        break;
      case MIME.PARSE_STATUS.PART_BODY:
        body[j].body += line;
        break;
    }
  }
  const { _, ...parts } = body;
  Object.keys(parts).forEach(n => {
    const part = parts[n];
    const { headers, body } = part;
    const { _: type } = headers['Content-Type'] || {};
    const { _: encoding } = headers['Content-Transfer-Encoding'] || {};
    part.raw = body;
    part.body = Buffer.from(body, encoding);
    switch(type){
      case 'text/plain':
      case 'text/html':
        part.body = body;
        break;
      case 'application/json':
        part.body = JSON.parse(body);
        break;
    }
  });
  return body;
};

module.exports = MIME;